import consola from 'consola';
import chalk from 'chalk';
import path from 'path';
import fs from 'fs';
import ora from 'ora';
import http from 'http';
import serveStatic from 'serve-static';
import { TwaGenerator, DigitalAssetLinks, TwaManifest, util, Config, JdkHelper, AndroidSdkTools, GradleWrapper, KeyTool, JarSigner } from '@bubblewrap/core';
import fs$1 from 'fs-extra';

var name = "nuxt-twa";
var version = "0.1.5";

async function buildAab(context) {
  const progressLogger = ora("Building AAB...").start();
  await context.tooling.gradleWrapper.bundleRelease();
  const unsignedAabPath = path.resolve(context.options.targetDir, "./app/build/outputs/bundle/release/app-release.aab");
  const destinationAabPath = path.resolve(context.options.targetDir, "./app-release-bundle.aab");
  if (context.keyConfig) {
    progressLogger.text = "Signing AAB...";
    await context.tooling.jarSigner.sign(context.keyConfig, context.keyConfig.password, context.keyConfig.keypassword, unsignedAabPath, destinationAabPath);
  } else {
    await fs.promises.copyFile(unsignedAabPath, destinationAabPath);
  }
  progressLogger.stop();
  consola.success(`AAB built ${context.keyConfig ? "and" : chalk.bold.yellow("NOT")} signed`);
}

async function buildApk(context) {
  const progressLogger = ora("Building APK...").start();
  await context.tooling.gradleWrapper.assembleRelease();
  const unsignedApkPath = path.resolve(context.options.targetDir, "./app/build/outputs/apk/release/app-release-unsigned.apk");
  if (context.keyConfig) {
    progressLogger.text = "Signing APK...";
    const signedApkPath = path.resolve(context.options.targetDir, "./app-release-signed.apk");
    await context.tooling.androidSdkTools.apksigner(context.keyConfig.path, context.keyConfig.password, context.keyConfig.alias, context.keyConfig.keypassword, unsignedApkPath, signedApkPath);
  } else {
    const destinationApkPath = path.resolve(context.options.targetDir, "./app-release-unsigned.apk");
    await fs.promises.copyFile(unsignedApkPath, destinationApkPath);
  }
  progressLogger.stop();
  consola.success(`APK built ${context.keyConfig ? "and" : chalk.bold.yellow("NOT")} signed`);
}

async function generateAndroidProject(context) {
  const server = http.createServer((req, res) => serveStatic(context.nuxtOptions.generate.dir)(req, res, null)).listen(3e3);
  await new TwaGenerator().createTwaProject(context.options.targetDir, context.twaManifest, null);
  server.close();
  await fs.promises.copyFile(path.resolve(__dirname, "../assets/Android.gitignore"), path.resolve(context.options.targetDir, ".gitignore"));
  consola.success("Android project generated");
}

async function generateAssetLinks(context) {
  if (!context.keyConfig) {
    return consola.warn("No Asset Links file generated because keys are not configured.");
  }
  const digitalAssetLinks = DigitalAssetLinks.generateAssetLinks(context.twaManifest.packageId, (await context.tooling.keyTool.keyInfo(context.keyConfig)).fingerprints.get("SHA256"));
  const linksPath = path.resolve(context.nuxtOptions.generate.dir, ".well-known/assetlinks.json");
  await fs$1.mkdirp(path.dirname(linksPath), null);
  await fs$1.promises.writeFile(linksPath, digitalAssetLinks);
  consola.success("Asset links emitted");
}

async function generateTwaManifest(context) {
  const twaManifest = await TwaManifest.fromWebManifestJson(new URL("http://localhost:3000" + context.nuxtOptions.pwa._manifestMeta.href), context.nuxtOptions.manifest);
  twaManifest.host = context.options.host;
  twaManifest.startUrl = context.nuxtOptions.manifest.start_url;
  twaManifest.packageId = context.options.packageId || util.generatePackageId(context.options.host);
  twaManifest.generatorApp = "nuxt-twa";
  context.twaManifest = twaManifest;
}

async function getKeyConfig(context) {
  const keyStorePath = process.env.KEYSTORE_PATH;
  const keyStorePassword = process.env.KEYSTORE_PASSWORD;
  const keyStoreKey = process.env.KEYSTORE_KEY;
  const keyStoreKeyPassword = process.env.KEYSTORE_KEY_PASSWORD;
  if (!keyStorePath || !keyStorePassword || !keyStoreKey || !keyStoreKeyPassword) {
    consola.warn("Missing or invalid key store config, Android build will NOT be signed and no asset links will be generated.");
  } else {
    context.keyConfig = {
      path: keyStorePath,
      alias: keyStoreKey,
      keypassword: keyStoreKeyPassword,
      password: keyStorePassword
    };
  }
}

async function setupTooling(context) {
  const androidSdkPath = context.options.androidSdkPath || process.env.ANDROID_HOME || process.env.ANDROID_SDK_ROOT;
  if (!androidSdkPath) {
    throw new Error("No android SDK path found. Aborting Android generation");
  }
  const jdkPath = (context.options.jdkPath || process.env.JAVA_HOME || "").replace(/\/Contents\/Home\/?/, "");
  if (!jdkPath) {
    throw new Error("No Java JDK path found. Aborting Android generation");
  }
  const config = new Config(jdkPath, androidSdkPath);
  const jdkHelper = new JdkHelper(process, config);
  const androidSdkTools = await AndroidSdkTools.create(process, config, jdkHelper);
  const gradleWrapper = new GradleWrapper(process, androidSdkTools, context.options.targetDir);
  const keyTool = new KeyTool(jdkHelper);
  const jarSigner = new JarSigner(jdkHelper);
  context.tooling = {
    androidSdkTools,
    gradleWrapper,
    jarSigner,
    keyTool
  };
}

const tasks = {
  buildAab,
  buildApk,
  generateAndroidProject,
  generateAssetLinks,
  generateTwaManifest,
  getKeyConfig,
  setupTooling
};

function buildContext(moduleOptions, nuxtOptions) {
  return {
    options: {
      ...{ targetDir: path.join(nuxtOptions.srcDir, "android") },
      ...moduleOptions
    },
    nuxtOptions
  };
}

async function twa(moduleOptions) {
  const options = {
    ...this.options.twa,
    ...moduleOptions
  };
  const context = buildContext(options, this.nuxt.options);
  try {
    await Promise.all([
      tasks.getKeyConfig(context),
      tasks.generateTwaManifest(context),
      tasks.setupTooling(context)
    ]);
  } catch (e) {
    return consola.error(e);
  }
  this.nuxt.hook("generate:done", async (_) => {
    consola.info(`Starting TWA generation
      	packageId: ${chalk.bold.green(context.twaManifest.packageId)}
      	host: ${chalk.bold.green(context.twaManifest.host)}`);
    await tasks.generateAndroidProject(context);
    try {
      await Promise.all([
        tasks.buildApk(context),
        tasks.buildAab(context)
      ]);
    } catch (e) {
      consola.error(e.stdout ? e.stdout : e);
    }
    await tasks.generateAssetLinks(context);
  });
}
twa.meta = {
  name,
  version
};

export { twa as default };
